package com.wifioferta;

import android.app.ListFragment;
import android.app.ProgressDialog;
import android.content.Context;
import android.net.wifi.p2p.WifiP2pConfig;
import android.net.wifi.p2p.WifiP2pDevice;
import android.net.wifi.p2p.WifiP2pDeviceList;
import android.net.wifi.p2p.WifiP2pManager.PeerListListener;
import android.os.Bundle;
import android.util.Base64;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.TextView;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;

import com.wifioferta.R;


/*
 * Fragment displaying found and matched offers 
 * from other users
 */
public class DeviceListFragment extends ListFragment implements
    PeerListListener {
  private static final String CATEGORY = "Kategoria :";
  private static final String TITLE = "Tytu :";
  private static final String DESCRIPTION = "Opis :";
  private static final String PRICE = "Cena :";
  private static final String NO_OFFERS = "Brak ofert";
  private List<WifiP2pDevice> peers = new ArrayList<WifiP2pDevice>();
  ProgressDialog progressDialog = null;
  View mContentView = null;
  private WifiP2pDevice device;

  @Override
  public void onActivityCreated(Bundle savedInstanceState) {
    super.onActivityCreated(savedInstanceState);
    this.setListAdapter(new WiFiPeerListAdapter(getActivity(),
        R.layout.row_devices, peers));

  }

  @Override
  public View onCreateView(LayoutInflater inflater, ViewGroup container,
      Bundle savedInstanceState) {
    mContentView = inflater.inflate(R.layout.device_list, null);
    return mContentView;
  }

  /**
   * @return this device
   */
  public WifiP2pDevice getDevice() {
    return device;
  }

  private static String getDeviceStatus(int deviceStatus) {
    Log.d(WiFiOfertaActivity.TAG, "Peer status :" + deviceStatus);
    switch (deviceStatus) {
      case WifiP2pDevice.AVAILABLE:
        return "Available";
      case WifiP2pDevice.INVITED:
        return "Invited";
      case WifiP2pDevice.CONNECTED:
        return "Connected";
      case WifiP2pDevice.FAILED:
        return "Failed";
      case WifiP2pDevice.UNAVAILABLE:
        return "Unavailable";
      default:
        return "Unknown";

    }
  }

  @Override
  public void onListItemClick(ListView l, View v, int position, long id) {
    WifiP2pDevice device = (WifiP2pDevice) getListAdapter().getItem(position);
    ((DeviceActionListener) getActivity()).showDetails(device);
  }

  /**
   * Array adapter for ListFragment that maintains WifiP2pDevice list.
   */
  public class WiFiPeerListAdapter extends ArrayAdapter<WifiP2pDevice> {

    private List<WifiP2pDevice> items;

    /**
     * @param context
     * @param textViewResourceId
     * @param objects
     */
    public WiFiPeerListAdapter(Context context, int textViewResourceId,
        List<WifiP2pDevice> objects) {
      super(context, textViewResourceId, objects);
      items = objects;

    }

    @Override
    public View getView(int position, View convertView, ViewGroup parent) {
      View v = convertView;
      if (v == null) {
        LayoutInflater vi = (LayoutInflater) getActivity().getSystemService(
            Context.LAYOUT_INFLATER_SERVICE);
        v = vi.inflate(R.layout.row_devices, null);
      }
      WifiP2pDevice device = items.get(position);
      if (device != null) {

        TextView top = (TextView) v.findViewById(R.id.device_name);
        TextView bottom = (TextView) v.findViewById(R.id.device_details);
        if (top != null) {
          getActivity();
          top.setText(decrypt(device.deviceName));
        }
        if (bottom != null) {
          bottom.setText(getDeviceStatus(device.status));
        }
      }

      return v;

    }
  }

  /**
   * Update UI for this device.
   * 
   * @param device
   *          WifiP2pDevice object
   */
  public void updateThisDevice(WifiP2pDevice device) {
    TextView view = (TextView) mContentView.findViewById(R.id.my_name);

    WiFiOfertaActivity activity = (WiFiOfertaActivity) getActivity();
    List<Sale> sales = activity.getOffers().getSales();
    for (Sale sale : sales) {
      if (sale.isActive()) {
        String name = TITLE + sale.getTittle() + "\n" + CATEGORY
            + sale.getCategory().getName() + " "
            + sale.getCategory().getParam1() + ": " + sale.getParam1() + " "
            + sale.getCategory().getParam2() + ": " + sale.getParam2() + "\n"
            + PRICE + sale.getPrice() + " " + DESCRIPTION
            + sale.getDescription();
        view.setText(name);
        ((DeviceActionListener) getActivity()).sendDesc(name);
      } else {
        view.setText(NO_OFFERS);
      }
    }

  }

  @Override
  public void onPeersAvailable(WifiP2pDeviceList peerList) {
    if (progressDialog != null && progressDialog.isShowing()) {
      progressDialog.dismiss();
    }
    peers.clear();
    WiFiOfertaActivity activity = (WiFiOfertaActivity) getActivity();
    List<Buy> buyes = activity.getOffers().getBuys();
    for (Buy buy : buyes) {
      for (WifiP2pDevice dev : peerList.getDeviceList()) {
        if (buy.isActive()
            && decrypt(dev.deviceName).startsWith(
                "+O/" + buy.getCategory().getId())
            && check(buy, dev.deviceName))
          peers.add(dev);
      }
    }
    ((WiFiPeerListAdapter) getListAdapter()).notifyDataSetChanged();
    if (peers.size() == 0) {
      return;
    }

  }

  private boolean check(Buy buy, String deviceName) {
    String offer = decrypt(deviceName);
    if (offer.contains("+")) {
      int start = offer.indexOf("+");
      int stop = offer.indexOf("+", start);
      if (stop == -1)
        stop = offer.indexOf("/", start);
      if (stop == -1)
        stop = offer.length();
      Double price = Double.parseDouble(offer.substring(start, stop));
      if (price > buy.getPrice())
        return false;
      stop = offer.indexOf("+", start);
      if (stop > start) {
        Double param1 = Double.parseDouble(offer.substring(start, stop));
        if (param1 != buy.getParam1())
          return false;
      }
      stop = offer.indexOf("/", start);
      if (stop > start) {
        String param2 = offer.substring(start, stop);
        if (!param2.matches(buy.getParam2()))
          return false;
      }
    }
    return true;
  }

  public void clearPeers() {
    peers.clear();
    ((WiFiPeerListAdapter) getListAdapter()).notifyDataSetChanged();
  }

  private String decrypt(String name) {
    byte[] unencryptedByteArray = null;

    String test = null;
    unencryptedByteArray = (Base64.decode(name, 0));
    try {
      test = new String(unencryptedByteArray, "US-ASCII");
    } catch (UnsupportedEncodingException e) {
      e.printStackTrace();
    }

    return test;

  }

  public interface DeviceActionListener {

    void showDetails(WifiP2pDevice device);

    void sendDesc(String data);

    void cancelDisconnect();

    void connect(WifiP2pConfig config);

    void disconnect();
  }
}
