package com.wifioferta;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.StreamCorruptedException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import com.wifioferta.R;
import com.wifioferta.DeviceListFragment.DeviceActionListener;

import android.app.Activity;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.net.wifi.p2p.WifiP2pConfig;
import android.net.wifi.p2p.WifiP2pDevice;
import android.net.wifi.p2p.WifiP2pManager;
import android.net.wifi.p2p.WifiP2pManager.ActionListener;
import android.net.wifi.p2p.WifiP2pManager.Channel;
import android.net.wifi.p2p.WifiP2pManager.ChannelListener;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.os.Messenger;
import android.provider.Settings;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.Toast;


/*
* Main activity of application. From this activity user can 
* search for other offers and connect with devices containing them
* This activity displays current sale offer of user
*/
public class WiFiOfertaActivity extends Activity implements ChannelListener,
    DeviceActionListener {

  public static final String TAG = "com.wifioferta";
  private static final String FILENAME = "MyOffers.txt";
  private WifiP2pManager manager;
  private boolean isWifiP2pEnabled = false;
  private boolean retryChannel = false;
  private Offers offers;

  private final IntentFilter intentFilter = new IntentFilter();
  private Channel channel;
  private BroadcastReceiver receiver = null;
  public Handler messageHandler = new MessageHandler();
  private Intent intentService;

  public class MessageHandler extends Handler {
    @Override
    public void handleMessage(Message message) {
      int state = message.arg1;
      switch (state) {
        case 1:
          Log.d(TAG, "handle 1 " + message.arg2);
          break;
        case 2:
          Log.d(TAG, "handle 2 " + message.arg2);
          break;
      }
    }
  }

  public void setIsWifiP2pEnabled(boolean isWifiP2pEnabled) {
    this.isWifiP2pEnabled = isWifiP2pEnabled;
  }

  @Override
  public void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    setContentView(R.layout.main);

    // add necessary intent values to be matched.

    intentFilter.addAction(WifiP2pManager.WIFI_P2P_STATE_CHANGED_ACTION);
    intentFilter.addAction(WifiP2pManager.WIFI_P2P_PEERS_CHANGED_ACTION);
    intentFilter.addAction(WifiP2pManager.WIFI_P2P_CONNECTION_CHANGED_ACTION);
    intentFilter.addAction(WifiP2pManager.WIFI_P2P_THIS_DEVICE_CHANGED_ACTION);

    manager = (WifiP2pManager) getSystemService(Context.WIFI_P2P_SERVICE);
    channel = manager.initialize(this, getMainLooper(), null);

    if (loadListFromFile() != null) {
      offers = loadListFromFile();
      intentService = new Intent(getBaseContext(), PeersSearchService.class);
      Bundle bundle = new Bundle();
      bundle.putParcelable("offers", offers);

      intentService.putExtras(bundle);
      intentService.putExtra("MESSENGER", new Messenger(messageHandler));

    } else {
      List<Buy> buyes = new ArrayList<Buy>();
      List<Sale> sales = new ArrayList<Sale>();
      offers = new Offers(sales, buyes);
    }
    intentService = new Intent(getBaseContext(), PeersSearchService.class);
    Bundle bundle = new Bundle();
    bundle.putParcelable("offers", offers);

    intentService.putExtras(bundle);
    intentService.putExtra("MESSENGER", new Messenger(messageHandler));

  }

  private Offers loadListFromFile() {
    Offers offer = offers;
    FileInputStream fis;
    try {
      fis = openFileInput(FILENAME);
      ObjectInputStream is = new ObjectInputStream(fis);
      offer = (Offers) is.readObject();
      is.close();
    } catch (FileNotFoundException e) {
      e.printStackTrace();
    } catch (StreamCorruptedException e) {
      e.printStackTrace();
    } catch (IOException e) {
      e.printStackTrace();
    } catch (ClassNotFoundException e) {
      e.printStackTrace();
    }

    return offer;
  }

  private void writeListToFile(Offers offer) {

    FileOutputStream fos;
    try {
      fos = openFileOutput(FILENAME, Context.MODE_PRIVATE);
      ObjectOutputStream os = new ObjectOutputStream(fos);
      os.writeObject(offer);
      os.close();
    } catch (FileNotFoundException e) {
      e.printStackTrace();
    } catch (IOException e) {
      e.printStackTrace();
    }

  }

  public void setName(String name) {

    Class<?> classToInvestigate = manager.getClass();

    try {
      Method[] aClassMethods = classToInvestigate.getDeclaredMethods();
      for (Method m : aClassMethods) {
        m.setAccessible(true);
        if (m.getName().equals("setDeviceName")) {
          // Getting this methhod directly caused application crash
          String deviceName = name;
          m.invoke(manager, channel, deviceName, null);
        }
      }
    } catch (IllegalAccessException e) {
      e.printStackTrace();
    } catch (IllegalArgumentException e) {
      e.printStackTrace();
    } catch (InvocationTargetException e) {
      e.printStackTrace();
    }
  }

  /** register the BroadcastReceiver with the intent values to be matched */
  @Override
  public void onResume() {
    super.onResume();
    receiver = new WiFiBroadcastReceiver(manager, channel, this);
    registerReceiver(receiver, intentFilter);
  }

  @Override
  public void onPause() {
    super.onPause();
    unregisterReceiver(receiver);
  }

  @Override
  public void onActivityResult(int requestCode, int resultCode, Intent data) {
    super.onActivityResult(requestCode, resultCode, data);
    if (requestCode == 0) {
      if (resultCode == RESULT_OK) {
        Bundle bundle = data.getExtras();

        offers = (Offers) bundle.getParcelable("offers");

        writeListToFile(offers);
        intentService = new Intent(getBaseContext(), PeersSearchService.class);
        Bundle sendBundle = new Bundle();
        sendBundle.putParcelable("offers", offers);

        intentService.putExtras(sendBundle);
        intentService.putExtra("MESSENGER", new Messenger(messageHandler));

        startService(intentService);
        if (!offers.getSales().isEmpty()) {
          for (Sale sale : offers.getSales()) {
            if (sale.isActive()) {
              String name = sale.getCodedName();
              setName(name);
              break;
            }
          }
        }
      }
    }
  }

  /**
   * Remove all peers and clear all fields. This is called on BroadcastReceiver
   * receiving a state change event.
   */
  public void resetData() {
    DeviceListFragment fragmentList = (DeviceListFragment) getFragmentManager()
        .findFragmentById(R.id.frag_list);
    DeviceDetailFragment fragmentDetails = (DeviceDetailFragment) getFragmentManager()
        .findFragmentById(R.id.frag_detail);
    if (fragmentList != null) {
      fragmentList.clearPeers();
    }
    if (fragmentDetails != null) {
      fragmentDetails.resetViews();
    }
  }

  @Override
  public boolean onCreateOptionsMenu(Menu menu) {
    MenuInflater inflater = getMenuInflater();
    inflater.inflate(R.menu.action_items, menu);
    return true;
  }

  @Override
  public boolean onOptionsItemSelected(MenuItem item) {
    switch (item.getItemId()) {
      case R.id.atn_direct_enable:
        if (manager != null && channel != null) {
          startActivityForResult(new Intent(Settings.ACTION_WIFI_SETTINGS), 1);
        } else {
          Log.e(TAG, "channel or manager is null");
        }
        return true;
      case R.id.atn_direct_stop:
        stopService(intentService);
        return true;
      case R.id.atn_direct_discover:
        if (!isWifiP2pEnabled) {
          Toast.makeText(WiFiOfertaActivity.this, R.string.p2p_off_warning,
              Toast.LENGTH_SHORT).show();
          return true;
        }
        intentService = new Intent(getBaseContext(), PeersSearchService.class);
        Bundle sendBundle = new Bundle();
        sendBundle.putParcelable("offers", offers);

        intentService.putExtras(sendBundle);
        intentService.putExtra("MESSENGER", new Messenger(messageHandler));

        startService(intentService);
        return true;
      default:
        return super.onOptionsItemSelected(item);
    }
  }

  @Override
  public void showDetails(WifiP2pDevice device) {
    DeviceDetailFragment fragment = (DeviceDetailFragment) getFragmentManager()
        .findFragmentById(R.id.frag_detail);
    fragment.showDetails(device);

  }

  @Override
  public void connect(final WifiP2pConfig config) {
    manager.connect(channel, config, new ActionListener() {

      @Override
      public void onSuccess() {
        Log.d("connect", "onsucces");
        Toast.makeText(WiFiOfertaActivity.this, "Connect success",
            Toast.LENGTH_SHORT).show();
      }

      @Override
      public void onFailure(int reason) {
        Toast.makeText(WiFiOfertaActivity.this, "Connect failed. Retry.",
            Toast.LENGTH_SHORT).show();
      }
    });
  }

  @Override
  public void disconnect() {
    final DeviceDetailFragment fragment = (DeviceDetailFragment) getFragmentManager()
        .findFragmentById(R.id.frag_detail);
    fragment.resetViews();
    manager.removeGroup(channel, new ActionListener() {

      @Override
      public void onFailure(int reasonCode) {
        Log.d(TAG, "Disconnect failed. Reason :" + reasonCode);

      }

      @Override
      public void onSuccess() {
        fragment.getView().setVisibility(View.GONE);
      }

    });
  }

  @Override
  public void onChannelDisconnected() {
    // we will try once more
    if (manager != null && !retryChannel) {
      Toast.makeText(this, "Channel lost. Trying again", Toast.LENGTH_LONG)
          .show();
      resetData();
      retryChannel = true;
      manager.initialize(this, getMainLooper(), this);
    } else {
      Toast
          .makeText(
              this,
              "Severe! Channel is probably lost premanently. Try Disable/Re-Enable P2P.",
              Toast.LENGTH_LONG).show();
    }
  }

  @Override
  public void cancelDisconnect() {


    if (manager != null) {
      final DeviceListFragment fragment = (DeviceListFragment) getFragmentManager()
          .findFragmentById(R.id.frag_list);
      if (fragment.getDevice() == null
          || fragment.getDevice().status == WifiP2pDevice.CONNECTED) {
        disconnect();
      } else if (fragment.getDevice().status == WifiP2pDevice.AVAILABLE
          || fragment.getDevice().status == WifiP2pDevice.INVITED) {

        manager.cancelConnect(channel, new ActionListener() {

          @Override
          public void onSuccess() {
            Toast.makeText(WiFiOfertaActivity.this, "Aborting connection",
                Toast.LENGTH_SHORT).show();
          }

          @Override
          public void onFailure(int reasonCode) {
            Toast.makeText(WiFiOfertaActivity.this,
                "Connect abort request failed. Reason Code: " + reasonCode,
                Toast.LENGTH_SHORT).show();
          }
        });
      }
    }

  }

  public void showOffer(View view) {
    stopService(intentService);
    Intent intent = new Intent(this, OffersManagerActivity.class);
    Bundle bundle = new Bundle();
    bundle.putParcelable("offers", offers);

    intent.putExtras(bundle);
    startActivityForResult(intent, 0);

  }

  public Offers getOffers() {
    return offers;
  }

  @Override
  public void sendDesc(String data) {
    DeviceDetailFragment fragment = (DeviceDetailFragment) getFragmentManager()
        .findFragmentById(R.id.frag_detail);
    fragment.sendDesc(data);

  }

}
